Regular commit
authorCarl N Baldwin <cnb@plane.(none)>
Thu, 20 Oct 2005 04:14:18 +0000 (22:14 -0600)
committerCarl N Baldwin <cnb@plane.(none)>
Thu, 20 Oct 2005 04:14:18 +0000 (22:14 -0600)
configure.ac
filedata.cpp
filedata.hpp
find-cmd.sh
main.cc

index af4793228f65d06d19a118440eaeeede6cb4ae83..3c21c101d7634c0e5af4cd0780672ad41b134168 100644 (file)
@@ -4,8 +4,6 @@ AC_COPYRIGHT([(c) Carl Baldwin.  All Rights Reserved. ])
 
 AM_INIT_AUTOMAKE([foreign -Wall])
 
-PKG_CHECK_MODULES([SQLITE3], [sqlite3])
-
 AC_PROG_CXX()
 AC_PROG_INSTALL()
 AC_PROG_RANLIB()
index cd82b5408396861ee9c95e4fc8be02e27c04992b..2a9840a7756717aec032566d93f1fc92f16b73b4 100644 (file)
@@ -1,4 +1,6 @@
 #include <string>
+#include <vector>
+#include <iostream>
 
 #include "filedata.hpp"
 
@@ -10,14 +12,71 @@ FileData::FileData( char               _type,
                     string             _group,
                     unsigned long long _size,
                     unsigned long long _modified_date,
-                    string             _name,
-                    unsigned long long _last_backup )
+                    unsigned long long _last_backup,
+                    string             _name )
 : filetype( _type ),
   permissions( _permissions ),
   username( _user ),
   groupname( _group ),
   filesize( _size ),
   modified_date( _modified_date ),
-  filename( _name ),
-  last_backup_date( _last_backup )
+  last_backup_date( _last_backup ),
+  filename( _name )
 {}
+
+vector<string> split( const string &line, char c, int limit = -1 ) {
+  string::size_type start = 0, end = 0;
+
+  vector<string> out;
+  while( 0 != limit-- && end != line.size() ) {
+    if( 0 == limit ) {
+      end = line.size();
+    } else {
+      end = line.find( c, start );
+      if( end == string::npos ) {
+        end = line.size();
+      }
+    }
+    out.push_back( line.substr( start, end-start ) );
+    start = end + 1;
+  }
+  return out;
+}
+
+ostream &operator<<( const FileData &d, ostream &o ) {
+  o << d.getFileType()       << ' ';
+  o << d.getPermissions()    << ' ';
+  o << d.getUserName()       << ' ';
+  o << d.getGroupName()      << ' ';
+  o << d.getFileSize()       << ' ';
+  o << d.getModifiedDate()   << ' ';
+  o << d.getLastBackupDate() << ' ';
+  o << d.getFileName()       << '\0';
+
+  return o;
+}
+
+istream &operator>>( istream &i, FileData &d ) {
+  string file_string;
+
+  for( int c = i.get(); 0 != c && ! i.eof(); c = i.get() ) {
+    file_string.push_back( c );
+  }
+
+  if( 0 != file_string.size() ) {
+    // Example entry
+    // type perms user group size datemodified name (8 total)
+    // f 0600 cnb cnb 424 20051015205340 0 ./.git/index
+    vector<string> vals = split( file_string, ' ', 8 );
+    d.setFileType(       vals[0][0] );
+    d.setPermissions(    vals[1] );
+    d.setUserName(       vals[2] );
+    d.setGroupName(      vals[3] );
+    d.setFileSize(       atoi( vals[4].c_str() ) );
+    d.setModifiedDate(   atoi( vals[5].c_str() ) );
+    d.setLastBackupDate( atoi( vals[6].c_str() ) );
+    d.setFileName(       vals[7] );
+  }
+
+  return i;
+}
index 3df81d9378772e0cbab5e5ca9bc17fb6f6a4226f..47a2690c069dc36c41dde635c86d4f6bbd5ad0b7 100644 (file)
@@ -8,14 +8,15 @@
 class FileData {
   public:
     // Construct a FileData object with default values
+    FileData() {}
     FileData( char,
               std::string,
               std::string,
               std::string,
               unsigned long long,
               unsigned long long,
-              std::string,
-              unsigned long long = 0
+              unsigned long long,
+              std::string
               );
 
     char               getFileType()       const { return filetype; }
@@ -37,7 +38,6 @@ class FileData {
     void setLastBackupDate( unsigned long long arg ) { last_backup_date = arg; }
 
   private:
-    FileData();
     FileData( const FileData & );
 
     char               filetype;
@@ -46,11 +46,14 @@ class FileData {
     std::string        groupname;
     unsigned long long filesize;
     unsigned long long modified_date;
-    std::string        filename;
-
     unsigned long long last_backup_date;
+    std::string        filename;
 };
 
+std::ostream &operator<<( const FileData &d, std::ostream &o );
+
+std::istream &operator>>( std::istream &i, FileData &d );
+
 struct FileDataLastBackupCmp {
   bool operator()( const FileData *a, const FileData *b ) {
     return a->getLastBackupDate() < b->getLastBackupDate();
index 71972b6d30cd28e690463506ced9ab4e1b0ee79b..006531c96fc4e797be01f3717ee832c5986c60e7 100755 (executable)
@@ -2,6 +2,6 @@
 
 {
   for type in d f l; do
-    find . -type $type -printf "$type %#m %g %u %s %CY%Cm%Cd%CH%CM%CS %p\0"
+    find . -type $type -printf "$type %#m %g %u %s %CY%Cm%Cd%CH%CM%CS %p\0"
   done
-} > list
+}
diff --git a/main.cc b/main.cc
index 1a1b9e7314cf73f0a01ff75ca50ed8efd0736e91..50bf7035cea3568d2ee2634907d13acee6ea720d 100644 (file)
--- a/main.cc
+++ b/main.cc
 #include <iostream>
+#include <fstream>
 #include <iterator>
 #include <algorithm>
 #include <cassert>
 
-#include <sqlite3.h>
-
 #include "filedata.hpp"
 
 using namespace std;
 
-vector<string> split( const string &line, char c, int limit = -1 ) {
-  string::size_type start = 0, end = 0;
-
-  vector<string> out;
-  while( 0 != limit-- && end != line.size() ) {
-    if( 0 == limit ) {
-      end = line.size();
-    } else {
-      end = line.find( c, start );
-      if( end == string::npos ) {
-        end = line.size();
-      }
-    }
-    out.push_back( line.substr( start, end-start ) );
-    start = end + 1;
-  }
-  return out;
-}
-
-// Callback function for getting files from the database
-int populate_set( void *files_v, int, char **vals, char ** ) {
-  file_set *files = reinterpret_cast<file_set*>( files_v );
-  files->insert( new FileData( vals[0][0],
-        vals[1],
-        vals[2],
-        vals[3],
-        atoi( vals[4] ),
-        atoi( vals[5] ),
-        vals[6]) );
-  return 0;
-}
-
-int main() {
-  string file_string;
-
-  file_set current;
-
-  // Parse the list of files on stdin
+template<class ISTREAM, class SET>
+void populate_set( ISTREAM &in, SET &files ) {
   do {
-    file_string.clear();
-    for( int c = cin.get(); 0 != c && ! cin.eof(); c = cin.get() ) {
-      file_string.push_back( c );
-    }
-    if( 0 != file_string.size() ) {
-      // Example entry
-      // type perms user group size datemodified name (7 total)
-      // f 0600 cnb cnb 424 20051015205340 ./.git/index
-      vector<string> vals = split( file_string, ' ', 7 );
-      current.insert( new FileData( vals[0][0],
-            vals[1],
-            vals[2],
-            vals[3],
-            atoi( vals[4].c_str() ),
-            atoi( vals[5].c_str() ),
-            vals[6]) );
-    }
-  } while( ! cin.eof() );
-
-  // Get the list of previously backed up files from the database.
-  sqlite3 *db;
-
-  const char *dbname = "test.db";
-  int rc = sqlite3_open( dbname, &db );
-  assert( SQLITE_OK == rc );
-
-  char *sqliteErrMsg = 0;
-  file_set backed_up;
-  rc = sqlite3_exec( db, "select * from filedata;", populate_set, &backed_up, &sqliteErrMsg );
-  assert( SQLITE_OK == rc );
-
-  rc = sqlite3_close( db );
-  assert( SQLITE_OK == rc );
+    FileData *data = new FileData();
+    in >> (*data);
+    files.insert( data );
+  } while( ! in.eof() );
+}
 
-  // Now divide the two sets into three sets (new, deleted and updated )
+template<class SET>
+void partition_sets( const SET &current, const SET &old,
+                     SET &added, SET &common, SET &deleted  ) {
   FileDataNameCmp cmp;
 
-  file_set added;
-  set_difference( current.begin(),   current.end(),
-                  backed_up.begin(), backed_up.end(),
+  set_difference( current.begin(), current.end(),
+                  old.begin(),     old.end(),
                   inserter( added, added.begin() ),
                   cmp );
 
-  file_set deleted;
-  set_difference( backed_up.begin(), backed_up.end(),
-                  current.begin(),   current.end(),
+  set_difference( old.begin(),     old.end(),
+                  current.begin(), current.end(),
                   inserter( deleted, deleted.begin() ),
                   cmp );
 
-  file_set updated;
-  set_union( current.begin(),   current.end(),
-             backed_up.begin(), backed_up.end(),
-             inserter( updated, updated.begin() ),
-             cmp );
+  set_union(      current.begin(), current.end(),
+                  old.begin(),     old.end(),
+                  inserter( common, common.begin() ),
+                  cmp );
+}
+
+int main() {
+  // Parse the list of current files on stdin
+  file_set current;
+  populate_set( cin, current );
+
+  file_set backed_up;
+  ifstream db( "test.db" );
+  populate_set( db, backed_up);
+
+  // Now divide the two sets into three sets (added, deleted and common )
+  file_set added, deleted, common;
+  partition_sets( current, backed_up, added, common, deleted );
 
   { // This little block will copy the last_backup_date from the second set to the first
+    FileDataNameCmp cmp;
+
     file_set updated_mirror;
     set_union( current.begin(),   current.end(),
                backed_up.begin(), backed_up.end(),
@@ -111,8 +61,8 @@ int main() {
                cmp );
 
     // TODO Now we need to copy the last_backup_date from 
-    file_set::iterator i = updated.begin(), j = updated_mirror.begin();
-    for( ; i != updated.end(); ++i, ++j ) {
+    file_set::iterator i = common.begin(), j = updated_mirror.begin();
+    for( ; i != common.end(); ++i, ++j ) {
       (*i)->setLastBackupDate( (*j)->getLastBackupDate() );
     }
   }
@@ -120,11 +70,11 @@ int main() {
   // Now find the list of files to backup.
   file_set backup_set;
 
-  // backup all new files
+  // backup all added files
   copy( added.begin(), added.end(), inserter( backup_set, backup_set.begin() ) );
 
-  // backup already backed-up files that have changed since the last backup date.
-  for( file_set::iterator i = updated.begin(); i != updated.end(); ++i ) {
+  // backup common files that have changed since the last backup date.
+  for( file_set::iterator i = common.begin(); i != common.end(); ++i ) {
     if( (*i)->getLastBackupDate() < (*i)->getModifiedDate() ) {
       backup_set.insert( *i );
     }
@@ -141,6 +91,8 @@ int main() {
 
   unsigned long long block_size = 512ULL;
 
+  bool finished = true;
+
   // Copy files over until full or out of files
   file_vector::reverse_iterator i = backups_bysize.rbegin();
   while( 0 != bytes_available && i != backups_bysize.rend() ) {
@@ -150,6 +102,8 @@ int main() {
     if( blocks <= bytes_available ) {
       bytes_available -= blocks;
       final_set.insert( *i );
+    } else {
+      finished = false;
     }
     ++i;
   }
@@ -157,6 +111,7 @@ int main() {
   // Now, sort the non-backed-up list by last_backup_date and back-fill
   if( 0 != bytes_available ) {
     file_vector leftovers;
+    FileDataNameCmp cmp;
     set_difference( current.begin(),   current.end(),
                     final_set.begin(), final_set.end(),
                     back_inserter( leftovers ),
@@ -181,16 +136,16 @@ int main() {
     }
   }
 
+  // TODO Get 'now' from time clock
   unsigned long long now = 20051019211200ULL;
   for( file_set::iterator k = final_set.begin(); k != final_set.end(); ++k ) {
     (*k)->setLastBackupDate( now );
   }
 
-  // Clean out the database
-  rc = sqlite3_exec( db, "delete from filedata;", NULL, NULL, &sqliteErrMsg );
-  assert( SQLITE_OK == rc );
-  rc = sqlite3_exec( db, "vacuum;", NULL, NULL, &sqliteErrMsg );
-  assert( SQLITE_OK == rc );
+  // Write the 'current' list to the dbfile
+  // Write the 'final_set' list to stdout
+
+  // If ! finished then write a flag to /tmp
 
   // Clean-up
   for( file_set::iterator i = backed_up.begin(); i != backed_up.end(); ++i ) { delete *i; }